# Computed field

The Computed field module allows administrative users to add fields to entity
types, where the fields' values are generated dynamically.

Developers can create computed field plugins that generate the dynamic values
for the fields. Plugins can optionally specify in code on which entity types and
bundles to create fields.

Computed fields can use the same field formatters as normal stored fields,
according to their data type.

## Requirements

This module requires no modules outside of Drupal core.

The following core patches are needed for Views to work:

- Automatically declare computed base fields to Views: https://www.drupal.org/project/drupal/issues/3349739.
- Allow computed bundle fields to work in Views: https://www.drupal.org/project/drupal/issues/2981047

Computed bundle fields will beed to be declared to Views manually; there is no
core patch for this yet.

## Installation

Install as you would normally install a contributed Drupal module. For further
information, see
[Installing Drupal Modules](https://www.drupal.org/docs/extending-drupal/installing-drupal-modules).

## Configuration

1. Go to the field admin page for the entity type and bundle you want to add a
   computed field to.
2. Click 'Add computed field'.
3. Enter the field label.
4. Select the computed field plugin to provide the field value. This may have
   additional configuration.
5. Save the form.
6. Configure the display options for the computed field in the same way as a
   normal stored field.

## Developers

The value of a computed field is generated by a computed field plugin. Creating
a computed field in the admin UI creates a config entity of type
'computed_field', similarly to how creating a normal stored field creates a
'field_config' config entity. The computed_field config entity holds the
computed field plugin ID and its configuration, if there is any.

A computed field plugin defines:
  - The field type, e.g. 'string' or 'entity_reference'. These are the same
    field types as used by normal stored fields, which allows existing field
    formatters to be used.
  - The code which returns the field value.
  - The cacheability of the field values. If a computed field's value is dynamic
    or depends on other things than the entity it is on, the output of the field
    can be rendered with a lazy builder. See the test module
    test_computed_field_plugins for examples of different cacheability.

Computed field plugins can either automatically attach fields to entity types,
or be configured to attach in the admin UI using a computed field config entity.

### Automatically attached plugins

The plugin's definition automatically declares computed fields. The plugin
annotation declares:
  - Which entity types it is attached to.
  - The scope: whether it is a base field or a bundle field.
  - Which bundles it is attached to in the latter case.

There is no configuration for automatically attaching plugins. If the plugin is
doing double duty as a configured attaching plugin, and has configuration, it
must supply default values for the configuration.

To prevent an automatically attached plugin from showing in the UI for computed
fields, set its no_ui annotation property.

See the test module test_computed_field_automatic for examples of these.

### Configured attaching plugins

Site admins create computed fields, using this plugin as the source of the field
value. The plugin may be configurable, and for this it must implement both of:
  - \Drupal\Component\Plugin\ConfigurableInterface
  - \Drupal\Core\Plugin\PluginFormInterface

A computed field created by creating a computed field entity is always a bundle
field, not a base field.

See the test module test_computed_field_plugins for examples of these.
